require 'geocoder'

module CoreModels
  class Base::Address < ApplicationRecord
    belongs_to :owner,   polymorphic: true, optional: true
    belongs_to :city,    class_name: 'CoreModels::Base::City', optional: true
    belongs_to :country, class_name: 'CoreModels::I18n::Country'

    has_many :telephone_numbers, class_name: 'CoreModels::Base::TelephoneNumber', as: :owner
    accepts_nested_attributes_for :telephone_numbers, reject_if: :all_blank, allow_destroy: true

    has_many :email_addresses, class_name: 'CoreModels::Base::EmailAddress', as: :owner
    accepts_nested_attributes_for :email_addresses, reject_if: :all_blank, allow_destroy: true

    has_many :web_addresses, class_name: 'CoreModels::Base::WebAddress', as: :owner
    accepts_nested_attributes_for :web_addresses, reject_if: :all_blank, allow_destroy: true

    # ::Gmaps4rails::ActsAsGmappable::ClassMethods.acts_as_gmappable validation: false
    # acts_as_gmappable validation: false unless Rails.env == 'test'
    geocoded_by         :gmaps4rails_address # can also be an IP address
    reverse_geocoded_by :latitude, :longitude
    unless Rails.env.test? || !(ENV.fetch('CoreModels_DISABLE_GEOCODING', nil) && ENV['CoreModels_DISABLE_GEOCODING'].empty?)
      after_validation :geocode
    end # auto-fetch coordinates

    # attr_accessible :room, :building, :address_line_1, :address_line_2, :postal_code, :town, :city, :region
    # attr_accessible :email_1, :email_2, :email_3, :url_1, :url_2, :url_3
    # attr_accessible :telephone_number_1, :telephone_number_2, :telephone_number_3
    # attr_accessible :fax_number_1, :fax_number_2

    # attr_accessible :latitude, :longitude

    # if ::Rails.env.test?
    # attr_accessible :country, :country_id
    # attr_accessible :owner, :owner_id
    # end

    # attr_accessible :world_coordinates

    validates :address_line_1, presence: true
    # validates :town, presence: true
    # validates :city, presence: true
    # validates :region, presence: true
    # validates :postal_code, presence: true
    # validates :country, presence: true

    validate  :city_and_country

    #----------------------------------------------------------------------------

    def to_s
      summary
    end

    def summary
      summary_en
    end

    def summary_en
      output = building.presence || ''
      output += building.blank? ? address_line_1 : ', ' + address_line_1
      output += ', ' + address_line_2 if address_line_2.present?
      # output += ', ' + self.town.best_name_value unless self.town.blank?
      output += ', ' + town if town.present?
      output += ', ' + city.name if city.present?
      # output += ', ' + self.region.best_name_value unless self.region.blank?
      output += ', ' + region if region.present?
      output += ', ' + country.best_name if country.present?
      output
    end

    # Formatted addess string suitable for Google Maps query
    def gmaps4rails_address
      output = building.presence || ''
      output += building.blank? ? address_line_1 : ', ' + address_line_1
      output += ', ' + address_line_2 if address_line_2.present?
      # output += ', ' + self.town.best_name_value('en') unless self.town.blank?
      output += ', ' + town if town.present?
      output += ', ' + city.name if city.present?
      # output += ', ' + self.region.best_name_value('en') unless self.region.blank?
      output += ', ' + region if region.present?
      output += ', ' + postal_code if postal_code.present?
      output += ', ' + country.best_name if country.present?
      output
    end

    #----------------------------------------------------------------------------

    private

    def city_and_country
      # Validate one or both of city and country are provided
      errors.add('At least one of city or country must be provided') if city.blank? && country.blank?

      # Inject country, if only the city is provided
      self.country = city.country if city.present? && country.blank?

      # Validate referential integrity between city/country
      return unless city.present? && country.present? && !city.country == country

      errors.add(:city, :invalid,
                 message: "does not belong to the country '#{country.best_name}'")
    end
  end
end
